#!/bin/bash
#Copyright 20130804 Matthew Headlee <mmh@matthewheadlee.com> released under GPLv3
#Minor bug fixes 20150206
usage() {
	echo "Usage: ${0} <Art Source> <Destination>" >&2
	exit 1
}
control_c() {
	cleanup
	echo -e "[\e[31;1mScript Aborted\e[m]"
	exit 1
}
cleanup() {
	#Removes temp files created during the scripts execution.
	#Disable for easier troubleshooting.
	rm -f "${SRCTREE}" "${DSTTREE}"
}

trap control_c SIGINT

##TODO: Add check to ensure convert/imagemagic is available.
##TODO: Add option for change thumbnail size from 64
#src = /mnt/ss-tng_music/
#dst = /mnt/xenixc/

[ ${#} != 2 ] && usage
SOURCE=${1}
if [ ! -d "${SOURCE}" ]; then
	echo "Error: Invalid Art Source Directory." >&2
	usage
fi
DESTINATION=${2}
if [ ! -d "${DESTINATION}" ]; then
	echo "Error: Invalid Destination Directory." >&2
	usage
fi

#This regex is used to identify multi-disc albums, must be kept compatible with sed -r and grep -E
MULTIALBUMREGEX='(Disc|CD|Disk)[[:space:]]*(One|Two|Three|Four|Five|Six|[0-9])$'

#Generate tmpfile for source tree so we don't have to run find a gajillion times.
echo -en "[\e[33;1mStatus\e[m] Building Directory Source Tree Listing...\033[0K\r"
SRCTREE=$(mktemp -t "aam-src.XXXXX")
find "${SOURCE}" -type d 1> "${SRCTREE}" 2> /dev/null
echo -e "[\e[32;1mDONE\e[m] Building Directory Source Tree Listing.\033[0K"

#Generate tmpfile for dest. tree so arg list doesn't become too large.
echo -en "[\e[33;1mStatus\e[m] Building Directory Destination Tree Listing...\033[0K\r"
DSTTREE=$(mktemp -t "aam-dst.XXXXX")
find "${DESTINATION}" -type d 1> "${DSTTREE}" 2> /dev/null
echo -e "[\e[32;1mDONE\e[m] Building Directory Destination Tree Listing.\033[0K"

#Change IFS to deal with files/dirs with spaces.
#OIFS=$IFS
#IFS=$'\n'
##Since we are no longer using a for-loop for this, IFS change is no longer required.
##^^ That needs to be verified.
while read F; do
	echo -en "[Status] Looking for '${F}'\033[0K\r"
	SSF_BASE=$(basename "$(echo ${F} | sed -r s/${MULTIALBUMREGEX}//i)")

	##This DOESN'T work well if different artists use same album name.
	##But fixing this is a lot more effort than I wish to put into this to have a tiny thumbnail on my player.
	SSF=$(grep -i "${SSF_BASE}" "${SRCTREE}" | head -n 1)
	if [ -z "${SSF}" ]; then
		echo -e "[\e[31;1mError\e[m] SSF is blank on F=${F}"
		continue
	fi

	#Maxdepth should normally be 1, except for multi-disc albums.
	MAXDEPTH=1
	if echo "${F}" | grep -E "${MULTIALBUMREGEX}" &> /dev/null; then
		MAXDEPTH=2
	fi
	SSFF=$(find "${SSF}" -maxdepth "${MAXDEPTH}" -type f -iname "folder.*" | head -n 1)

	##TODO: rewrite and test: if [ -z "${SSFF}" -a -f "${SSFF}" ];
	if [ -f "${SSFF}" ] && [ "${SSFF}" != "" ] ; then
		echo -en "[\e[33;1mConverting\e[m] ${SSFF}\033[0K\r"
			#Should we really continue on an error? Usually a cp fail is due to windows SMB going stupid which requires a reboot on windows.
			#http://support.microsoft.com/kb/177078
			#http://linux.derkeiler.com/Newsgroups/comp.os.linux.networking/2006-10/msg00629.html

			##MUCH Better articles about this problem:
			#http://alan.lamielle.net/2009/09/03/windows-7-nonpaged-pool-srv-error-2017
			#http://www.decuslib.com/decus/vmslt99a/nt/tips.txt
		if ! cp -f "${SSFF}" "${F}/"; then
			echo -e "[\e[31;1mCP FAILED\e[m] ${SSFF}\033[0K"
			continue
		fi
		if ! convert -size x64 -sampling-factor 2x1 -thumbnail x64 -crop 64x64+0+0 -adaptive-sharpen 5 "${SSFF}" "${F}/cover.jpg" &> /dev/null; then
			echo -e "[\e[31;1mCONVERT FAILED\e[m] ${SSFF}\033[0K"
			continue
		fi

		echo -e "[\e[32;1mDONE\e[m] ${SSFF}\033[0K"
	fi
done < "${DSTTREE}"
#IFS=$OIFS

cleanup
echo -e "[\e[32;1mCOMPLETE\e[m]"
